@@ -39,19 +39,16 @@ from Crypto.SelfTest.st_common import list_test_cases
 
 from Crypto.Util.py3compat import *
 
-from Crypto.Math.Numbers import Integer as IntegerGeneric
-from Crypto.Math import _Numbers_int as NumbersInt
+from Crypto.Math._IntegerNative import IntegerNative
 
 
 class TestIntegerBase(unittest.TestCase):
 
     def setUp(self):
-        if not hasattr(self, "Integer"):
-            from Crypto.Math.Numbers import Integer as IntegerDefault
-            self.Integer = IntegerDefault
+        pass
 
     def Integers(self, *arg):
-        return list(map(self.Integer, arg))
+        return map(self.Integer, arg)
 
     def test_init_and_equality(self):
         Integer = self.Integer
@@ -70,13 +67,13 @@ class TestIntegerBase(unittest.TestCase):
         self.assertEqual(v4, 10 ** 10)
         self.assertEqual(v5, -10 ** 10)
 
-        self.assertFalse(v1 == v4)
+        self.failIf(v1 == v4)
 
         # Init and comparison between Integer's
         v6 = Integer(v1)
         self.assertEqual(v1, v6)
 
-        self.assertFalse(Integer(0) == None)
+        self.failIf(Integer(0) == None)
 
     def test_conversion_to_int(self):
         v1, v2 = self.Integers(-23, 2 ** 1000)
@@ -85,17 +82,17 @@ class TestIntegerBase(unittest.TestCase):
 
     def test_equality_with_ints(self):
         v1, v2, v3 = self.Integers(23, -89, 2 ** 1000)
-        self.assertTrue(v1 == 23)
-        self.assertTrue(v2 == -89)
-        self.assertFalse(v1 == 24)
-        self.assertTrue(v3 == 2 ** 1000)
+        self.failUnless(v1 == 23)
+        self.failUnless(v2 == -89)
+        self.failIf(v1 == 24)
+        self.failUnless(v3 == 2 ** 1000)
 
     def test_conversion_to_str(self):
         v1, v2, v3, v4 = self.Integers(20, 0, -20, 2 ** 1000)
-        self.assertTrue(str(v1) == "20")
-        self.assertTrue(str(v2) == "0")
-        self.assertTrue(str(v3) == "-20")
-        self.assertTrue(str(v4) == "10715086071862673209484250490600018105614048117055336074437503883703510511249361224931983788156958581275946729175531468251871452856923140435984577574698574803934567774824230985421074605062371141877954182153046474983581941267398767559165543946077062914571196477686542167660429831652624386837205668069376")
+        self.failUnless(str(v1) == "20")
+        self.failUnless(str(v2) == "0")
+        self.failUnless(str(v3) == "-20")
+        self.failUnless(str(v4) == "10715086071862673209484250490600018105614048117055336074437503883703510511249361224931983788156958581275946729175531468251871452856923140435984577574698574803934567774824230985421074605062371141877954182153046474983581941267398767559165543946077062914571196477686542167660429831652624386837205668069376")
 
     def test_repr(self):
         v1, v2 = self.Integers(-1, 2**80)
@@ -120,7 +117,7 @@ class TestIntegerBase(unittest.TestCase):
         Integer = self.Integer
 
         v1 = Integer.from_bytes(b("\x00"))
-        self.assertTrue(isinstance(v1, Integer))
+        self.failUnless(isinstance(v1, Integer))
         self.assertEqual(0, v1)
 
         v2 = Integer.from_bytes(b("\x00\x00"))
@@ -132,81 +129,81 @@ class TestIntegerBase(unittest.TestCase):
     def test_inequality(self):
         # Test Integer!=Integer and Integer!=int
         v1, v2, v3, v4 = self.Integers(89, 89, 90, -8)
-        self.assertTrue(v1 != v3)
-        self.assertTrue(v1 != 90)
-        self.assertFalse(v1 != v2)
-        self.assertFalse(v1 != 89)
-        self.assertTrue(v1 != v4)
-        self.assertTrue(v4 != v1)
-        self.assertTrue(self.Integer(0) != None)
+        self.failUnless(v1 != v3)
+        self.failUnless(v1 != 90)
+        self.failIf(v1 != v2)
+        self.failIf(v1 != 89)
+        self.failUnless(v1 != v4)
+        self.failUnless(v4 != v1)
+        self.failUnless(self.Integer(0) != None)
 
     def test_less_than(self):
         # Test Integer<Integer and Integer<int
         v1, v2, v3, v4, v5 = self.Integers(13, 13, 14, -8, 2 ** 10)
-        self.assertTrue(v1 < v3)
-        self.assertTrue(v1 < 14)
-        self.assertFalse(v1 < v2)
-        self.assertFalse(v1 < 13)
-        self.assertTrue(v4 < v1)
-        self.assertFalse(v1 < v4)
-        self.assertTrue(v1 < v5)
-        self.assertFalse(v5 < v1)
+        self.failUnless(v1 < v3)
+        self.failUnless(v1 < 14)
+        self.failIf(v1 < v2)
+        self.failIf(v1 < 13)
+        self.failUnless(v4 < v1)
+        self.failIf(v1 < v4)
+        self.failUnless(v1 < v5)
+        self.failIf(v5 < v1)
 
     def test_less_than_or_equal(self):
         # Test Integer<=Integer and Integer<=int
         v1, v2, v3, v4, v5 = self.Integers(13, 13, 14, -4, 2 ** 10)
-        self.assertTrue(v1 <= v1)
-        self.assertTrue(v1 <= 13)
-        self.assertTrue(v1 <= v2)
-        self.assertTrue(v1 <= 14)
-        self.assertTrue(v1 <= v3)
-        self.assertFalse(v1 <= v4)
-        self.assertTrue(v1 <= v5)
-        self.assertFalse(v5 <= v1)
+        self.failUnless(v1 <= v1)
+        self.failUnless(v1 <= 13)
+        self.failUnless(v1 <= v2)
+        self.failUnless(v1 <= 14)
+        self.failUnless(v1 <= v3)
+        self.failIf(v1 <= v4)
+        self.failUnless(v1 <= v5)
+        self.failIf(v5 <= v1)
 
     def test_more_than(self):
         # Test Integer>Integer and Integer>int
         v1, v2, v3, v4, v5 = self.Integers(13, 13, 14, -8, 2 ** 10)
-        self.assertTrue(v3 > v1)
-        self.assertTrue(v3 > 13)
-        self.assertFalse(v1 > v1)
-        self.assertFalse(v1 > v2)
-        self.assertFalse(v1 > 13)
-        self.assertTrue(v1 > v4)
-        self.assertFalse(v4 > v1)
-        self.assertTrue(v5 > v1)
-        self.assertFalse(v1 > v5)
+        self.failUnless(v3 > v1)
+        self.failUnless(v3 > 13)
+        self.failIf(v1 > v1)
+        self.failIf(v1 > v2)
+        self.failIf(v1 > 13)
+        self.failUnless(v1 > v4)
+        self.failIf(v4 > v1)
+        self.failUnless(v5 > v1)
+        self.failIf(v1 > v5)
 
     def test_more_than_or_equal(self):
         # Test Integer>=Integer and Integer>=int
         v1, v2, v3, v4 = self.Integers(13, 13, 14, -4)
-        self.assertTrue(v3 >= v1)
-        self.assertTrue(v3 >= 13)
-        self.assertTrue(v1 >= v2)
-        self.assertTrue(v1 >= v1)
-        self.assertTrue(v1 >= 13)
-        self.assertFalse(v4 >= v1)
+        self.failUnless(v3 >= v1)
+        self.failUnless(v3 >= 13)
+        self.failUnless(v1 >= v2)
+        self.failUnless(v1 >= v1)
+        self.failUnless(v1 >= 13)
+        self.failIf(v4 >= v1)
 
     def test_bool(self):
         v1, v2, v3, v4 = self.Integers(0, 10, -9, 2 ** 10)
         self.assertFalse(v1)
         self.assertFalse(bool(v1))
-        self.assertTrue(v2)
-        self.assertTrue(bool(v2))
-        self.assertTrue(v3)
-        self.assertTrue(v4)
+        self.failUnless(v2)
+        self.failUnless(bool(v2))
+        self.failUnless(v3)
+        self.failUnless(v4)
 
     def test_is_negative(self):
         v1, v2, v3, v4, v5 = self.Integers(-3 ** 100, -3, 0, 3, 3**100)
-        self.assertTrue(v1.is_negative())
-        self.assertTrue(v2.is_negative())
-        self.assertFalse(v4.is_negative())
-        self.assertFalse(v5.is_negative())
+        self.failUnless(v1.is_negative())
+        self.failUnless(v2.is_negative())
+        self.failIf(v4.is_negative())
+        self.failIf(v5.is_negative())
 
     def test_addition(self):
         # Test Integer+Integer and Integer+int
         v1, v2, v3 = self.Integers(7, 90, -7)
-        self.assertTrue(isinstance(v1 + v2, self.Integer))
+        self.failUnless(isinstance(v1 + v2, self.Integer))
         self.assertEqual(v1 + v2, 97)
         self.assertEqual(v1 + 90, 97)
         self.assertEqual(v1 + v3, 0)
@@ -216,7 +213,7 @@ class TestIntegerBase(unittest.TestCase):
     def test_subtraction(self):
         # Test Integer-Integer and Integer-int
         v1, v2, v3 = self.Integers(7, 90, -7)
-        self.assertTrue(isinstance(v1 - v2, self.Integer))
+        self.failUnless(isinstance(v1 - v2, self.Integer))
         self.assertEqual(v2 - v1, 83)
         self.assertEqual(v2 - 7, 83)
         self.assertEqual(v2 - v3, 97)
@@ -226,7 +223,7 @@ class TestIntegerBase(unittest.TestCase):
     def test_multiplication(self):
         # Test Integer-Integer and Integer-int
         v1, v2, v3, v4 = self.Integers(4, 5, -2, 2 ** 10)
-        self.assertTrue(isinstance(v1 * v2, self.Integer))
+        self.failUnless(isinstance(v1 * v2, self.Integer))
         self.assertEqual(v1 * v2, 20)
         self.assertEqual(v1 * 5, 20)
         self.assertEqual(v1 * -2, -8)
@@ -234,7 +231,7 @@ class TestIntegerBase(unittest.TestCase):
 
     def test_floor_div(self):
         v1, v2, v3 = self.Integers(3, 8, 2 ** 80)
-        self.assertTrue(isinstance(v1 // v2, self.Integer))
+        self.failUnless(isinstance(v1 // v2, self.Integer))
         self.assertEqual(v2 // v1, 2)
         self.assertEqual(v2 // 3, 2)
         self.assertEqual(v2 // -3, -3)
@@ -244,7 +241,7 @@ class TestIntegerBase(unittest.TestCase):
     def test_remainder(self):
         # Test Integer%Integer and Integer%int
         v1, v2, v3 = self.Integers(23, 5, -4)
-        self.assertTrue(isinstance(v1 % v2, self.Integer))
+        self.failUnless(isinstance(v1 % v2, self.Integer))
         self.assertEqual(v1 % v2, 3)
         self.assertEqual(v1 % 5, 3)
         self.assertEqual(v3 % 5, 1)
@@ -254,7 +251,7 @@ class TestIntegerBase(unittest.TestCase):
 
     def test_simple_exponentiation(self):
         v1, v2, v3 = self.Integers(4, 3, -2)
-        self.assertTrue(isinstance(v1 ** v2, self.Integer))
+        self.failUnless(isinstance(v1 ** v2, self.Integer))
         self.assertEqual(v1 ** v2, 64)
         self.assertEqual(pow(v1, v2), 64)
         self.assertEqual(v1 ** 3, 64)
@@ -267,7 +264,7 @@ class TestIntegerBase(unittest.TestCase):
     def test_modular_exponentiation(self):
         v1, v2, v3 = self.Integers(23, 5, 17)
 
-        self.assertTrue(isinstance(pow(v1, v2, v3), self.Integer))
+        self.failUnless(isinstance(pow(v1, v2, v3), self.Integer))
         self.assertEqual(pow(v1, v2, v3), 7)
         self.assertEqual(pow(v1, 5,  v3), 7)
         self.assertEqual(pow(v1, v2, 17), 7)
@@ -384,7 +381,7 @@ class TestIntegerBase(unittest.TestCase):
 
     def test_and(self):
         v1, v2, v3 = self.Integers(0xF4, 0x31, -0xF)
-        self.assertTrue(isinstance(v1 & v2, self.Integer))
+        self.failUnless(isinstance(v1 & v2, self.Integer))
         self.assertEqual(v1 & v2, 0x30)
         self.assertEqual(v1 & 0x31, 0x30)
         self.assertEqual(v1 & v3, 0xF0)
@@ -394,7 +391,7 @@ class TestIntegerBase(unittest.TestCase):
 
     def test_or(self):
         v1, v2, v3 = self.Integers(0x40, 0x82, -0xF)
-        self.assertTrue(isinstance(v1 | v2, self.Integer))
+        self.failUnless(isinstance(v1 | v2, self.Integer))
         self.assertEqual(v1 | v2, 0xC2)
         self.assertEqual(v1 | 0x82, 0xC2)
         self.assertEqual(v2 | v3, -0xD)
@@ -403,7 +400,7 @@ class TestIntegerBase(unittest.TestCase):
     def test_right_shift(self):
         v1, v2, v3 = self.Integers(0x10, 1, -0x10)
         self.assertEqual(v1 >> 0, v1)
-        self.assertTrue(isinstance(v1 >> v2, self.Integer))
+        self.failUnless(isinstance(v1 >> v2, self.Integer))
         self.assertEqual(v1 >> v2, 0x08)
         self.assertEqual(v1 >> 1, 0x08)
         self.assertRaises(ValueError, lambda: v1 >> -1)
@@ -426,22 +423,28 @@ class TestIntegerBase(unittest.TestCase):
             v4 = self.Integer(0x90)
             v4 >>= -1
         self.assertRaises(ValueError, l)
-        def m():
+        def m1():
             v4 = self.Integer(0x90)
             v4 >>= 2 ** 1000
-        self.assertRaises(ValueError, m)
+            return v4
+        self.assertEqual(0, m1())
+        def m2():
+            v4 = self.Integer(-1)
+            v4 >>= 2 ** 1000
+            return v4
+        self.assertEqual(-1, m2())
 
-    def test_left_shift(self):
+    def _test_left_shift(self):
         v1, v2, v3 = self.Integers(0x10, 1, -0x10)
         self.assertEqual(v1 << 0, v1)
-        self.assertTrue(isinstance(v1 << v2, self.Integer))
+        self.failUnless(isinstance(v1 << v2, self.Integer))
         self.assertEqual(v1 << v2, 0x20)
         self.assertEqual(v1 << 1, 0x20)
         self.assertEqual(v3 << 1, -0x20)
         self.assertRaises(ValueError, lambda: v1 << -1)
         self.assertRaises(ValueError, lambda: v1 << (2 ** 1000))
 
-    def test_in_place_right_shift(self):
+    def test_in_place_left_shift(self):
         v1, v2, v3 = self.Integers(0x10, 1, -0x10)
         v1 <<= 0
         self.assertEqual(v1, 0x10)
@@ -480,17 +483,17 @@ class TestIntegerBase(unittest.TestCase):
     def test_odd_even(self):
         v1, v2, v3, v4, v5 = self.Integers(0, 4, 17, -4, -17)
 
-        self.assertTrue(v1.is_even())
-        self.assertTrue(v2.is_even())
-        self.assertFalse(v3.is_even())
-        self.assertTrue(v4.is_even())
-        self.assertFalse(v5.is_even())
+        self.failUnless(v1.is_even())
+        self.failUnless(v2.is_even())
+        self.failIf(v3.is_even())
+        self.failUnless(v4.is_even())
+        self.failIf(v5.is_even())
 
-        self.assertFalse(v1.is_odd())
-        self.assertFalse(v2.is_odd())
-        self.assertTrue(v3.is_odd())
-        self.assertFalse(v4.is_odd())
-        self.assertTrue(v5.is_odd())
+        self.failIf(v1.is_odd())
+        self.failIf(v2.is_odd())
+        self.failUnless(v3.is_odd())
+        self.failIf(v4.is_odd())
+        self.failUnless(v5.is_odd())
 
     def test_size_in_bits(self):
         v1, v2, v3, v4 = self.Integers(0, 1, 0x100, -90)
@@ -510,18 +513,18 @@ class TestIntegerBase(unittest.TestCase):
 
     def test_perfect_square(self):
 
-        self.assertFalse(self.Integer(-9).is_perfect_square())
-        self.assertTrue(self.Integer(0).is_perfect_square())
-        self.assertTrue(self.Integer(1).is_perfect_square())
-        self.assertFalse(self.Integer(2).is_perfect_square())
-        self.assertFalse(self.Integer(3).is_perfect_square())
-        self.assertTrue(self.Integer(4).is_perfect_square())
-        self.assertTrue(self.Integer(39*39).is_perfect_square())
-        self.assertFalse(self.Integer(39*39+1).is_perfect_square())
+        self.failIf(self.Integer(-9).is_perfect_square())
+        self.failUnless(self.Integer(0).is_perfect_square())
+        self.failUnless(self.Integer(1).is_perfect_square())
+        self.failIf(self.Integer(2).is_perfect_square())
+        self.failIf(self.Integer(3).is_perfect_square())
+        self.failUnless(self.Integer(4).is_perfect_square())
+        self.failUnless(self.Integer(39*39).is_perfect_square())
+        self.failIf(self.Integer(39*39+1).is_perfect_square())
 
         for x in range(100, 1000):
-            self.assertFalse(self.Integer(x**2+1).is_perfect_square())
-            self.assertTrue(self.Integer(x**2).is_perfect_square())
+            self.failIf(self.Integer(x**2+1).is_perfect_square())
+            self.failUnless(self.Integer(x**2).is_perfect_square())
 
     def test_fail_if_divisible_by(self):
         v1, v2, v3 = self.Integers(12, -12, 4)
@@ -566,7 +569,7 @@ class TestIntegerBase(unittest.TestCase):
     def test_inverse(self):
         v1, v2, v3, v4, v5, v6 = self.Integers(2, 5, -3, 0, 723872, 3433)
 
-        self.assertTrue(isinstance(v1.inverse(v2), self.Integer))
+        self.failUnless(isinstance(v1.inverse(v2), self.Integer))
         self.assertEqual(v1.inverse(v2), 3)
         self.assertEqual(v1.inverse(5), 3)
         self.assertEqual(v3.inverse(5), 3)
@@ -586,7 +589,7 @@ class TestIntegerBase(unittest.TestCase):
 
     def test_gcd(self):
         v1, v2, v3, v4 = self.Integers(6, 10, 17, -2)
-        self.assertTrue(isinstance(v1.gcd(v2), self.Integer))
+        self.failUnless(isinstance(v1.gcd(v2), self.Integer))
         self.assertEqual(v1.gcd(v2), 2)
         self.assertEqual(v1.gcd(10), 2)
         self.assertEqual(v1.gcd(v3), 1)
@@ -595,7 +598,7 @@ class TestIntegerBase(unittest.TestCase):
 
     def test_lcm(self):
         v1, v2, v3, v4, v5 = self.Integers(6, 10, 17, -2, 0)
-        self.assertTrue(isinstance(v1.lcm(v2), self.Integer))
+        self.failUnless(isinstance(v1.lcm(v2), self.Integer))
         self.assertEqual(v1.lcm(v2), 30)
         self.assertEqual(v1.lcm(10), 30)
         self.assertEqual(v1.lcm(v3), 102)
@@ -669,37 +672,36 @@ class TestIntegerBase(unittest.TestCase):
 class TestIntegerInt(TestIntegerBase):
 
     def setUp(self):
-        self.Numbers = NumbersInt
-        self.Integer = NumbersInt.Integer
+        self.Integer = IntegerNative
         TestIntegerBase.setUp(self)
 
 
-class TestIntegerGeneric(unittest.TestCase):
+class testIntegerRandom(unittest.TestCase):
 
     def test_random_exact_bits(self):
 
         for _ in range(1000):
-            a = IntegerGeneric.random(exact_bits=8)
-            self.assertFalse(a < 128)
-            self.assertFalse(a >= 256)
+            a = IntegerNative.random(exact_bits=8)
+            self.failIf(a < 128)
+            self.failIf(a >= 256)
 
         for bits_value in range(1024, 1024 + 8):
-            a = IntegerGeneric.random(exact_bits=bits_value)
-            self.assertFalse(a < 2**(bits_value - 1))
-            self.assertFalse(a >= 2**bits_value)
+            a = IntegerNative.random(exact_bits=bits_value)
+            self.failIf(a < 2**(bits_value - 1))
+            self.failIf(a >= 2**bits_value)
 
     def test_random_max_bits(self):
 
         flag = False
         for _ in range(1000):
-            a = IntegerGeneric.random(max_bits=8)
+            a = IntegerNative.random(max_bits=8)
             flag = flag or a < 128
-            self.assertFalse(a>=256)
-        self.assertTrue(flag)
+            self.failIf(a>=256)
+        self.failUnless(flag)
 
         for bits_value in range(1024, 1024 + 8):
-            a = IntegerGeneric.random(max_bits=bits_value)
-            self.assertFalse(a >= 2**bits_value)
+            a = IntegerNative.random(max_bits=bits_value)
+            self.failIf(a >= 2**bits_value)
 
     def test_random_bits_custom_rng(self):
 
@@ -712,20 +714,20 @@ class TestIntegerGeneric(unittest.TestCase):
                 return bchr(0) * size
 
         custom_rng = CustomRNG()
-        a = IntegerGeneric.random(exact_bits=32, randfunc=custom_rng)
+        a = IntegerNative.random(exact_bits=32, randfunc=custom_rng)
         self.assertEqual(custom_rng.counter, 4)
 
     def test_random_range(self):
 
-        func = IntegerGeneric.random_range
+        func = IntegerNative.random_range
 
         for x in range(200):
             a = func(min_inclusive=1, max_inclusive=15)
-            self.assertTrue(1 <= a <= 15)
+            self.failUnless(1 <= a <= 15)
 
         for x in range(200):
             a = func(min_inclusive=1, max_exclusive=15)
-            self.assertTrue(1 <= a < 15)
+            self.failUnless(1 <= a < 15)
 
         self.assertRaises(ValueError, func, min_inclusive=1, max_inclusive=2,
                                             max_exclusive=3)
@@ -734,14 +736,13 @@ class TestIntegerGeneric(unittest.TestCase):
 def get_tests(config={}):
     tests = []
     tests += list_test_cases(TestIntegerInt)
-
+    
     try:
-        from Crypto.Math import _Numbers_gmp as NumbersGMP
+        from Crypto.Math._IntegerGMP import IntegerGMP
 
         class TestIntegerGMP(TestIntegerBase):
             def setUp(self):
-                self.Numbers = NumbersGMP
-                self.Integer = NumbersGMP.Integer
+                self.Integer = IntegerGMP
                 TestIntegerBase.setUp(self)
 
         tests += list_test_cases(TestIntegerGMP)
@@ -750,12 +751,11 @@ def get_tests(config={}):
         sys.stdout.write("Skipping GMP tests (%s)\n" % str(e) )
 
     try:
-        from Crypto.Math import _Numbers_custom as NumbersCustomModexp
+        from Crypto.Math._IntegerCustom import IntegerCustom
 
         class TestIntegerCustomModexp(TestIntegerBase):
             def setUp(self):
-                self.Numbers = NumbersCustomModexp
-                self.Integer = NumbersCustomModexp.Integer
+                self.Integer = IntegerCustom
                 TestIntegerBase.setUp(self)
 
         tests += list_test_cases(TestIntegerCustomModexp)
@@ -763,7 +763,7 @@ def get_tests(config={}):
         import sys
         sys.stdout.write("Skipping custom modexp tests (%s)\n" % str(e) )
 
-    tests += list_test_cases(TestIntegerGeneric)
+    tests += list_test_cases(testIntegerRandom)
     return tests
 
 if __name__ == '__main__':
